/* ADT.h
 * For GTKeyboard
 * written by David Allen s2mdalle@titan.vcu.edu
 * http://opop.nols.com/
 *
 * This file is released under the terms of the GNU General Public License.
 * Please see COPYING for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>
#include <assert.h>
#include <time.h>
#include <sys/time.h>

#include "user_shortcuts.h"

/* Maximum number of custom colors.  10 is a good number, above that is
 * really not needed per se.
 */
#define MAX_CUSTOM_COLORS         10

/**************************************************************************/

/* ADTs */
/* Program configuration settings */
typedef struct{
     /* Changed to bits in 0.97.3 to save a little space. */
     /* Different program options - initialized in app.c changed in 
      * file_foo.c and messed with elsewhere.
      *
      * See app.c and the initialization code for comments on what they 
      * all mean.
      */
     unsigned int REDIRECT_POLICY_IMPLICIT     :1;
     unsigned int STATUS_LOGGING               :1;
     unsigned int CAPS_LOCK                    :1;
     unsigned int SEND_TO_BOTH_WINDOWS         :1;
     unsigned int ASK_REMAP_ON_EXIT            :1;
     unsigned int BORDERLESS                   :1;
     unsigned int INFO_POPUPS                  :1;
     unsigned int SHIFT                        :1;
     unsigned int ALT_GR                       :1;
     unsigned int WORD_WRAP                    :1;
     unsigned int NUMLOCK                      :1;
     unsigned int ASK_SAVE_ON_EXIT             :1;
     unsigned int VERBOSE                      :1;
     unsigned int ALT                          :1;
     unsigned int CONTROL                      :1;
     unsigned int misc                         :1;
     unsigned int RANDOMIZE_XPMS               :1;
     unsigned int DISPLAY_TOOLBAR              :1;
     unsigned int DISPLAY_ROW_BUTTONS          :1;
     unsigned int SHOW_ALL_WINDOWS             :1;
     unsigned int SHOW_MENU                    :1;
     unsigned int USE_KEYBOARD                 :1;
     int RANDOM_STRING_CHARACTERS;
     
     void (*signal_handling)();   /* Function pointer for signal handling    */
     char *home;                  /* Home dir of user running the app.       */
     char *workingfile;           /* Current file. Name of file to save/open */
     char *tempslot;
     char *extrafiles;            /* Extra files, docs, default rcfile, lic. */
     char *xpmshare;              /* where xpm's are. $blah/share/gtkeyboard */
     char *redirect_window_name;  /* Window name of redirection location.    */
     char *szNeedle;              /* What was entered in the find window     */
     Window redirect_window;      /* Xlib window structure for redirection.  */
     Window other;                /* Temporary window structure for Xlib     */
     char *shortcuts[SHORTCUTS];  /* User defined shortcuts                  */

     int xpos;                    /* Startup position for the application */
     int ypos;
     int handlerid;               /* Timeout handlerID (follow_focus) */

     KEYBOARD *keyboard;
     KEYBOARD *last_keyboard;     

     /* These memory areas won't be used unless PROD isn't defined.
      * When PROD isn't defined, we're not debugging, and they essentially
      * keep track of the number of memory calls that I make.  See also
      * mem_header.h for a full definition of how they're used.
      */
#if defined(GTKEYBOARD_MEMORY_DEBUGGING) || defined(DEBUGGING)
     long MEM;             /* Number of bytes allocated          */
     int gnew;             /* Number of gnew() calls (alloc)     */
     int gnew0;            /* Number of gnew0() calls (alloc)    */
     int gmalloc;          /* Number of g_malloc() calls (alloc) */
     int gstrdup;          /* Number of g_strdup() calls (alloc) */
     int gfree;            /* Number of g_free() calls (free)    */
#endif
} GTKeyboardOptions;

typedef struct {
     unsigned int toolbar   : 1;
     unsigned int menubar   : 1;
     unsigned int keyboard  : 1;
     unsigned int status    : 1;
     unsigned int text      : 1;
     unsigned int buttonbar : 1;
} GTKeyboardElements;

/****************************************************************************/

typedef struct{
     Window xwindow;               /* So we can play with xlib. (our window) */
     GtkWidget *KEYBOARD;          /* Container containing all keyb elements */
     GtkWidget *menubar;
     GtkWidget *keyboard_socket;   /* Widget that keyboard gets stuck into   */
     GtkWidget *bottom_row_buttons;/* Self explanatory?                      */
     GtkWidget *mainbox;           /* main component, pack keyboard here     */
     GtkWidget *s_table;           /* Holds the status window (a table)      */
     GtkWidget *status;            /* Actually holds the status text window  */
     GtkWidget *toolbar;           /* Main toolbar in the main window        */
     GtkWidget *main_output_text;  /* Holds actual text widget               */
     GtkWidget *master_text;       /* Main text widget - (really a table).   */

     int BUMP_AMOUNT;              /* Value in pixels the window gets bumped */

     /* If a field has a comment of "Cleaned" next to it, then it is
      * either deallocated or destroyed in 
      * callbacks.c:gtkeyboard_mem_cleanup() via macro calls.
      */

     char *custom_colors[MAX_CUSTOM_COLORS];    /* Cleaned */
     gchar *fontname;                           /* Cleaned */
     gchar *kfontname;                          /* Cleaned */

     GtkShadowType SHADOW_TYPE;
     GtkPositionType PULLOFFS_SIDE;

     GtkItemFactory *item_factory;              /* Cleaned */
     GtkWidget *popup_menu;                     /* Cleaned */
     GdkCursor *cursor;
     GtkStyle *style;                           /* Cleaned */
     GtkStyle *deflt;                           /* Cleaned */
     GdkFont *font;                             /* Cleaned */
     GtkStyle *kstyle;                          /* Cleaned */
     GdkFont *kfont;                            /* Cleaned */
} GTKeyboardGUI;

/**************************************************************************/

/* ALL of these will be on by default. */
typedef struct{
     unsigned int TB_FONT             :1;
     unsigned int TB_KEYBOARD_FONT    :1;
     unsigned int TB_WINDOW_GRAB      :1;
     unsigned int TB_WINDOW_IGNORE    :1;
     unsigned int TB_RAISE            :1;
     unsigned int TB_LOWER            :1;
     unsigned int TB_CUT              :1;
     unsigned int TB_COPY             :1;
     unsigned int TB_PASTE            :1;
     unsigned int TB_NEW              :1;
     unsigned int TB_OPEN             :1;
     unsigned int TB_SAVE             :1;
     unsigned int TB_COLOR            :1;
     unsigned int TB_STATUS_REPORT    :1;
     unsigned int TB_RESET_STYLE      :1;
     unsigned int TB_SHORTCUTS        :1;
     unsigned int TB_POSITIONING      :1;
} GTKeyboardToolbar;

typedef struct{
     int width;
     int height;
     int x;
     int y;
} window_dimensions;

typedef struct{
     KeySym keysym;
     KeyCode keycode;
     char *string;
} gtkeyboard_keypress;

/* This one is only really needed for stylish.c */
typedef struct{
    GtkWidget *dialog;
    GdkColor *color;
} color_info;

