/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 ** irxevent.c
 *
 * Heinrich Langos  <heinrich@null.net>
 * small modifications by Christoph Bartelmus <columbus@hit.handshake.de>
 * Many, many modifications by David Allen <s2mdalle@titan.vcu.edu>
 * (bastardized for the evil purposes of GTKeyboard)
 *
 * for more information see the irxevent.README file
 */

#define IRXEVENT_C
#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "config.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <sys/time.h>

#include "master.h"

KeySym character_to_keysym(char i)
{
     KeySym x;
     char subbuf[15];
     
     if(isalpha(i))  /* Hopefully most common event */
     {
	  sprintf(subbuf,"%c",i);
	  x = XStringToKeysym(subbuf);
	  return(x);
     } /* End if */
     else if(isdigit(i))
     {
	  switch(i){
	  case '0': return(XK_0); case '1': return(XK_1); 
	  case '2': return(XK_2); case '3': return(XK_3);
	  case '4': return(XK_4); case '5': return(XK_5);
	  case '6': return(XK_6); case '7': return(XK_7);
	  case '8': return(XK_8); case '9': return(XK_9);
	  default:  return(XK_0); /* Shouldn't happen */
	  } /* End switch */
     } /* End if */
     
     switch(i){
     case '>':  return(XK_greater);   /* Cover likely HTML tag elements 1st */
     case '<':  return(XK_less);
     case '/':  return(XK_slash);
     case ' ':  return(XK_space);
     case '=':  return(XK_equal);
     case '+':  return(XK_plus);
     case '!':  return(XK_exclam);
     case '&':  return(XK_ampersand);
     case '(':  return(XK_parenleft);
     case ')':  return(XK_parenright);
     case '"':  return(XK_quotedbl);
     case '-':  return(XK_minus);
     case '*':  return(XK_asterisk);
     case '.':  return(XK_period);
     case '\b': return(XK_BackSpace);
     case '\t': return(XK_Tab);
     case '\n': return(XK_Return);
     default:   return((KeySym)i);
     } /* End switch */

     /* I have no idea, so we'll just cast it and hope we're right.  :) */
     return((KeySym)i);
} /* End character_to_keysym() */

/* seems that xfree86 computes the timestamps for events like this 
 * strange but it relies on the *1000-32bit-wrap-around     
 * if anybody knows exactly how to do it, please contact me 
 * returns: a timestamp for the 
 * constructed event 
 */
static Time fake_timestamp()
{
     int  tint;
     struct timeval  tv;
     struct timezone tz; /* is not used since ages */
     gettimeofday(&tv, &tz);
     tint = (int)tv.tv_sec * 1000;
     tint = tint/1000 * 1000;
     tint = tint + tv.tv_usec/1000;
     return((Time)tint);
} /* End fake_timestamp() */

/* in: id of the root window name of the target window 
 * returns: id of the target window 
 * called by: main 
 */
Window find_window(Window top,char *name)
{
     char *wname,*iname;
     XClassHint xch;
     Window *children,foo;
     int revert_to_return;
     unsigned int nc;
     if (!strcmp(active_window_name,name))
     {
	  XGetInputFocus(GDK_DISPLAY(), &foo, &revert_to_return);
	  return(foo);
     } /* End if */
     /* First the base case */
     if (XFetchName(GDK_DISPLAY(),top,&wname))
     {
	  if (!strncmp(wname,name,strlen(name)))  
	  {
	       XFree(wname);
	       return(top);  /* found it! */
	  }
	  XFree(wname);
     } /* End if */
     
     if(XGetIconName(GDK_DISPLAY(),top,&iname))
     {
	  if (!strncmp(iname,name,strlen(name)))  
	  {
	       XFree(iname);
	       return(top);  /* found it! */
	  }
	  XFree(iname);
     }

     if(XGetClassHint(GDK_DISPLAY(),top,&xch))  
     {
	  if(!strcmp(xch.res_class,name))  
	  {
	       XFree(xch.res_name); XFree(xch.res_class);
	       return(top);  /* found it! */
	  }
	  XFree(xch.res_name); XFree(xch.res_class);
     }
     
     if(!XQueryTree(GDK_DISPLAY(),top,&foo,&foo,&children,&nc) || 
	children==NULL) 
	  return(0);  /* no more windows here */
     
     /* check all the sub windows */
     for(;nc>0; nc--)  
     {
	  top = find_window(children[nc-1],name);
	  if(top) 
	       break;  /* we found it somewhere */
     }

     /* Free that mem!  Yeehaw!!! */
     if(children!=NULL) 
	  XFree(children);
     return(top);
}

/* in: keyname eg. CTRL-SHIFT-A (see README) 
 * in/out: XEvent that gets constructed 
 * calls: fake_timestamp 
 * called by: sendkey 
 */
int make_key(char *keyname, XKeyEvent *xev)
{
     /* Modified 9/6/99 by David Allen - the new version is
      * glib-spiffy, although I kept all of the old code hidden in 
      * extremely ugly #defines
      * This appears to be more stable and portable - email me if
      * you find something wrong
      * s2mdalle@titan.vcu.edu
      */

     KeySym converted;
     char *part2;
     char tpart2[128];
     struct keymodlist_t *kmlptr;
     char **garray;
     char *tmpstring = NULL;

     part2 = tpart2;
     xev->type = KeyPress;
     xev->display=GDK_DISPLAY();
     xev->root=root;
     xev->subwindow = None;
     xev->time=fake_timestamp();
     xev->x=1; xev->y=1;
     xev->x_root=1; xev->y_root=1;
     xev->same_screen = True;
     
     xev->state=0;

     /* Split the string completely */
     garray = g_strsplit(keyname, key_delimiter, -1);

     while((tmpstring = *garray++))
     {
	  part2 = strncpy(part2,tmpstring,128);

	  kmlptr=keymodlist;
	  while(kmlptr->name)
	  {
	       if(!g_strcasecmp(kmlptr->name, tmpstring))
		    xev->state|=kmlptr->mask;
	       kmlptr++;
	  } /* End while */
     } /* End while */

     /* Copious error checking. */
     converted = XStringToKeysym(part2);
     if(converted == NoSymbol || converted == (KeySym)NULL || !converted)
     {
	  fprintf(Q,"IRXEVENT Warning: make_key XStringToKeysym() NoSymbol\n");
	  fflush(Q);
	  return(0);
     } /* End if */

     xev->keycode = XKeysymToKeycode(GDK_DISPLAY(),converted);

     /* Error check the XKeysymToKeycode call */
     if(xev->keycode == 0)
     {
	  fprintf(Q,"IRXEVENT Warning: make_key XKeysymToKeycode() was 0\n");
	  fflush(Q);
	  return(0);
     } /* End if */

     return(1);
} /* End make_key */

void gtkeyboard_XEvent_common_setup(XKeyEvent *xev)
{
     xev->type         = KeyPress;
     xev->display      = GDK_DISPLAY();
     xev->root         = root;
     xev->subwindow    = None;
     xev->time         = fake_timestamp();
     xev->same_screen  = True;
     xev->state        = 0;
     xev->x = xev->y = xev->x_root = xev->y_root = 1;
} /* End gtkeyboard_XEvent_common_setup */

int setup_keycode(XKeyEvent *xev, KeySym sym)
{
     xev->keycode = XKeysymToKeycode(GDK_DISPLAY(), sym);

     /* Error check the XKeysymToKeycode call */
     ERROR_CHECK_KEYCODE(xev);
     return(1);
}

static int assign_keycode_from_keysym(KeySym foo, XKeyEvent *xev)
{
     xev->keycode = XKeysymToKeycode(GDK_DISPLAY(), foo);

     /* Check and assign masks. */
     if(options.SHIFT)
     {
	  xev->state |= ShiftMask;
	  options.SHIFT = 0;
     }
     if(options.CAPS_LOCK)
	  xev->state |= LockMask;
     if(options.CONTROL)
	  xev->state |= ControlMask;
     if(options.ALT)
	  xev->state |= Mod1Mask;
     if(options.NUMLOCK)
	  xev->state |= Mod2Mask;

     if(xev->keycode != 0)
	  return 1;
     else return 0;
} /* End assign_keycode_from_keysym() */

void keysym_sendkey(KeySym somesym, Window w)
{
     gtkeyboard_XEvent_common_setup((XKeyEvent *)&xev);

     /* assign_keycode_from_keysym() will also add in the needed
      * masks.  WARNING:  This may change options.SHIFT and other 
      * bitflags in options according to whether or not they should
      * change. 
      */
     if(!assign_keycode_from_keysym(somesym, (XKeyEvent *)&xev))
     {
	  fprintf(stderr,"keysym_sendkey() Error: 0 KeyCode from (%ld,'%s')\n",
		  somesym, XKeysymToString(somesym));
	  fflush(stderr);
	  
	  chocolate(zero_keycode);
	  annoying_popup(zero_keycode);
	  return;
     } /* End if */

     xev.xkey.window = w;

     /* This may produce a BadWindow error with Xlib.  Bummer. */

     gdk_error_trap_push();      /* Catch errors, hopefully */
     
     XSendEvent(GDK_DISPLAY(), w, True, KeyPressMask, &xev);

     xev.type = KeyRelease;
     /* usleep(50000);	*/
     XFlush(GDK_DISPLAY());
     xev.xkey.time = fake_timestamp();
     XSendEvent(GDK_DISPLAY(),w,True,KeyReleaseMask,&xev);
     XSync(GDK_DISPLAY(),True);

     gdk_flush();
     if(gdk_error_trap_pop())
     {
	  chocolate("XError:  Cannot redirect that key.\n");
     } /* End if */

     return;
} /* End keysym_sendkey() */

/* sends a keypress event 
 * waits for 0,2 seconds (changed by MDA to something else, can't remember)
 * and sends the keyrelease event
 * in: key name 
 * in: target window id 
 * out: nothing 
 * calls: make_key 
 * called by: main 
 */
void sendkey(char *keyname, Window w)
{
     KeySym conversion;
     char converted;
     if(w == (Window)NULL || !w)
     {
	  gdk_beep();
	  gtkeyboard_error(1,"Invalid window.  Does it still exist?\n");
	  gtkeyboard_message(2,"Error in sendkey():  GTKeyboard will not ",
			     "continue trying to send this instance.\n");
	  return;
     } /* End if */

     if(strlen(keyname) == 1)
     {
#ifdef DEBUGGING
	  fprintf(Q,"sendkey() strlen(keyname) == 1 doing shortcut...\n");
	  fflush(Q);
#endif /* DEBUGGING */
	  
	  gtkeyboard_XEvent_common_setup((XKeyEvent *)&xev);
	  converted = *keyname;
	  conversion = (KeySym) converted;

	  /* Catch keycode == 0 errors so we don't segfault or kill the
	   * application that we're redirecting into.
	   */
	  if(!setup_keycode((XKeyEvent *)&xev, conversion))
	  {
	       gtkeyboard_error(2,"There was an error making this key.\n",
				"Do you need to remap your keyboard?\n");
	       return;
	  }
     } /* End if */
     else
     {
	  if(!make_key(keyname ,(XKeyEvent *)&xev))
	  {
	       gtkeyboard_error(2,"There was an error making this key.\n",
				"Do you need to remap your keyboard?\n");
	       return;
	  } /* End if */

     } /* End else */

     xev.xkey.window=w;

     /* This may produce a BadWindow error with Xlib.  Bummer. */
     XSendEvent(GDK_DISPLAY(),w,True,KeyPressMask,&xev);

     xev.type = KeyRelease;
     /* usleep(200000); */
     /* change - (David Allen) let's try sleeping for 0.05 seconds */
     /* usleep(50000);	*/
     XFlush(GDK_DISPLAY());
     xev.xkey.time = fake_timestamp();
     XSendEvent(GDK_DISPLAY(),w,True,KeyReleaseMask,&xev);
     XSync(GDK_DISPLAY(),True);
     return;
} /* End sendkey */




