/* STYLISH.C
 *
 * For use with GTKeyboard
 * by David Allen
 * s2mdalle@titan.vcu.edu http://opop.nols.com/
 * Released under the terms of the GNU General Public License
 *
 * Styles/Themes/Colors/Fonts relating to GTKeyboard.  Manipulation,
 * creation, destruction, and all that other good stuff.
 */

/* Typedefs and external prototypes are included through master.h */
#define STYLISH_C
#ifndef MASTER_H
#  include "master.h"
#endif /* MASTER_H */
#ifndef STYLISH_H
#  include "include/stylish.h"
#endif /* STYLISH_H */

/* Static prototypes for this file */
static void GetDialogColor           (GdkColor *color);
static void color_ok_button          (GtkWidget *w, color_info *colorInfo);
static void CloseDialog              (GtkWidget *w, color_info *di);
static GtkStyle *CreateBGStyle       (GdkColor background);
static GtkStyle *new_style           (GdkColor c);
static void grabfont                 (GtkWidget *emitter, GtkWidget *fontsel);
static void keyboardgrabfont         (GtkWidget *emitter, GtkWidget *fontsel);

/* switch_cursor
 * Sets the window cursor to something that the user chooses
 * check out include/ui.h for a list of possible cursors in the
 * options->looks->cursors menu.  GDK provides enums for different 
 * cursors, which end up as the gint CURSOR param
 */
void switch_cursor(GtkWidget *emitter, gint CURSOR)
{
     GUI.cursor = gdk_cursor_new(CURSOR);
     gdk_window_set_cursor(window->window, GUI.cursor);
} /* End switch_cursor */

/*
 * CreateBGStyle 
 *
 * Creates a style using the colors passed in.  
 * Set the foreground color, the text color and the
 * background color.
 * Makes all states the same color
 */
static GtkStyle *CreateBGStyle(GdkColor background)
{
     GtkStyle *defstyle;
     GtkStyle *style;
     register int i;
     
     defstyle = gtk_widget_get_default_style ();
     style = gtk_style_copy (defstyle);

     /* Set the style in different places */
     for (i=0; i<5; i++) 
	  style->bg[i] = background;
     return (style);
} /* End CreateBGStyle */

/* Sets all of the windows and subwidgets back to the default style
 * which is stored in GUI.deflt.  Should be a macro...damn itemfactories.
 */
void reset_application_style(GtkWidget *emitter, gpointer data)
{
     /* Set the toplevel window and everything under it to the default
      * style 
      */
     SetStyleRecursively(window, GUI.deflt);
} /* End reset_application_style */

/*
 * Creates a random style using the list of colors
 * above.  There is no checking to see if the colors 
 * would be a bad choice.
 */
static GtkStyle *new_style(GdkColor c)
{
    GtkStyle *style;
    
    /* Create new style */
    style = CreateBGStyle (c);
    
    return (style);
} /* End new_style */

/*
 * SetStyleRecursively
 *
 * Set the style on the current widget and on any 
 * children widgets.
 */
void SetStyleRecursively (GtkWidget *widget, gpointer data)
{
     GtkStyle *style;
     
     style = (GtkStyle *) data;
     
     gtk_widget_set_style (widget, style);
     
     if (GTK_IS_CONTAINER (widget)) 
     {
	  gtk_container_foreach(GTK_CONTAINER(widget), 
				SetStyleRecursively, style);
     }
} /* End SetStyleRecursively */

/* color_button_clicked()
 * This is what happens when you click "Ok" in the color
 * selection dialog
 */
void color_button_clicked(GtkWidget *widget, gpointer gdata)
{
    GtkStyle *style;
    GdkColor color;

    /* --- Call the color dialog to get a color --- */
    GetDialogColor (&color);

    /* --- Create a style based on the color --- */
    style = new_style(color);

    /* Set the style of the wiget based on that new style 
     * We're setting properties for the WHOLE MAIN WINDOW
     */
    SetStyleRecursively (window, (gpointer)style);
} /* End color_button_clicked */

static void GetDialogColor (GdkColor *color)
{
     static GtkWidget *windowl = NULL;
     color_info *colorInfo;
     
     windowl = gtk_color_selection_dialog_new("Color Selection Dialog");
     
     /* Allocate memory for the colorinfo and populate it */
     colorInfo = g_malloc(sizeof (color_info));
     colorInfo->dialog = windowl;
     colorInfo->color = color;
     
     gtk_color_selection_set_opacity(GTK_COLOR_SELECTION(
	  GTK_COLOR_SELECTION_DIALOG(windowl)->colorsel),
				     TRUE);
     
     gtk_color_selection_set_update_policy(GTK_COLOR_SELECTION(
	  GTK_COLOR_SELECTION_DIALOG(windowl)->colorsel),
					   GTK_UPDATE_CONTINUOUS);
     
    gtk_signal_connect(GTK_OBJECT(windowl), "destroy",
		       GTK_SIGNAL_FUNC(smack),
		       windowl);

    gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(
	 windowl)->ok_button),
		       "clicked",
		       GTK_SIGNAL_FUNC (color_ok_button),
		       colorInfo);
    
    gtk_signal_connect(GTK_OBJECT(windowl), "destroy",
		       GTK_SIGNAL_FUNC (CloseDialog),
		       colorInfo);
    
    gtk_signal_connect_object(
	 GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG (windowl)->cancel_button),
	 "clicked",
	 GTK_SIGNAL_FUNC(gtk_widget_destroy),
	 GTK_OBJECT(windowl));
    
    gtk_widget_show(windowl);

    gtk_grab_add (windowl);
    gtk_main ();
} /* End GetDialogColor */

/* User decided on a color */
static void color_ok_button(GtkWidget *w, color_info *colorInfo)
{
     GtkColorSelectionDialog *cs;
     GtkColorSelection *colorsel;
     gdouble color[4];
     
     cs = (GtkColorSelectionDialog *) colorInfo->dialog;
     colorsel = GTK_COLOR_SELECTION (cs->colorsel);
     
     gtk_color_selection_get_color (colorsel, color);
     colorInfo->color->red = color[0] * 0xffff;
     colorInfo->color->green = color[1] * 0xffff;
     colorInfo->color->blue = color[2] * 0xffff;
     
     gtk_widget_destroy (GTK_WIDGET (cs));
} /* End color_ok_button */

static void CloseDialog(GtkWidget *w, color_info *di)
{
     gtk_main_quit ();
     gtk_grab_remove (w);
     g_free_(di);
} /* End close_dialog */

static void grabfont(GtkWidget *emitter, GtkWidget *fontsel)
{
     GtkFontSelectionDialog *fsd = GTK_FONT_SELECTION_DIALOG(fontsel);
     register int x=0;
     XFontStruct *fontOK = (XFontStruct *)NULL;

     GUI.fontname = gtk_font_selection_dialog_get_font_name (fsd);
     gtk_widget_destroy (fontsel);

     chocolate("Got ");
     chocolate(GUI.fontname);
     chocolate(" as our font.\n");

     /* Test load the font.  If it fails, fontOK is null, and get on with
      * life after giving the user an error.
      * If it is successful, immediately unload the font, since we're using
      * GTK+ here, mmmk?
      */
     fontOK = XLoadQueryFont(GDK_DISPLAY(), GUI.fontname);
     
     if(fontOK == (XFontStruct *)NULL || (!fontOK))
     {
	  g_warning("Error loading font %s\n",GUI.fontname);
	  gtkeyboard_error(3,"Can't find fontname ",GUI.fontname,
			   "in the system list of available fonts.  ",
			   "Sorry - font not loaded.\n");
	  return;
     }
     else
	  XFreeFont(GDK_DISPLAY(), fontOK);

     /* Else the font is available, load it and reset bg style color */

     GUI.font = gdk_font_load(GUI.fontname);
     g_free_(GUI.fontname);
     GUI.fontname = NULL;

     chocolate("Updating widget...(this may take a while)...\n");

     GUI.style       = gtk_style_copy(GUI.deflt);

     /* This should solve the background color copying problem */
     for(x=0; x<5; x++)
     {
	  GUI.style->bg[x] = GUI.deflt->bg[x];
     }

     GUI.style->font = GUI.font;
     
     gtk_widget_set_style(GUI.main_output_text, GUI.style);
} /* End grabfont */

static void keyboardgrabfont(GtkWidget *emitter, GtkWidget *fontsel)
{
     GtkFontSelectionDialog *fsd = GTK_FONT_SELECTION_DIALOG(fontsel);
     register int x=0;
     XFontStruct *fontOK = (XFontStruct *)NULL;

     GUI.kfontname = gtk_font_selection_dialog_get_font_name (fsd);
     gtk_widget_destroy (fontsel);

     gtkeyboard_message(3,"(KEYBOARD) Got ", GUI.kfontname, 
			" as our font.\n");

     fontOK = XLoadQueryFont(GDK_DISPLAY(), GUI.kfontname);
     
     if(fontOK == (XFontStruct *)NULL || (!fontOK))
     {
	  g_warning("Error loading font %s\n",GUI.kfontname);
	  gtkeyboard_error(3,"Can't find fontname ",GUI.kfontname,
			   "in the system list of available fonts.  ",
			   "Sorry - font not loaded.\n");
	  return;
     }
     else
	  XFreeFont(GDK_DISPLAY(), fontOK);

     /* Else the font is available, load it and reset bg style color */

     GUI.kfont = gdk_font_load(GUI.kfontname);
     g_free_(GUI.kfontname);
     GUI.kfontname = NULL;

     chocolate("Updating widget...(this may take a while)...\n");

     GUI.kstyle       = gtk_style_copy(GUI.deflt);

     /* This should solve the background color copying problem */
     for(x=0; x<5; x++)
     {
	  GUI.kstyle->bg[x] = GUI.deflt->bg[x];
     }

     GUI.kstyle->font = GUI.kfont;

     /* Set the keyboard style recursively */
     SetStyleRecursively(GUI.KEYBOARD, GUI.kstyle);
} /* End keyboardgrabfont */

void FontSelect(GtkWidget *emitter, gpointer data)
{
     GtkWidget *widget;
     GtkFontSelectionDialog *fontsel;

     widget = gtk_font_selection_dialog_new ("Grab-A-Font (TM)");
     fontsel = GTK_FONT_SELECTION_DIALOG (widget);
     
     gtk_signal_connect(GTK_OBJECT(fontsel->ok_button), "clicked",
			GTK_SIGNAL_FUNC (grabfont), fontsel);
     
     gtk_signal_connect_object(GTK_OBJECT(fontsel->cancel_button), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
			       GTK_OBJECT (fontsel));
     gtk_widget_show(widget);
} /* End FontSelect */

void KeyboardFontSelect(GtkWidget *emitter, gpointer data)
{
     GtkWidget *widget;
     GtkFontSelectionDialog *fontsel;

     widget = gtk_font_selection_dialog_new ("Grab-A-Font (TM)");
     fontsel = GTK_FONT_SELECTION_DIALOG (widget);
     
     gtk_signal_connect(GTK_OBJECT(fontsel->ok_button), "clicked",
			GTK_SIGNAL_FUNC (keyboardgrabfont), fontsel);
     
     gtk_signal_connect_object(GTK_OBJECT(fontsel->cancel_button), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
			       GTK_OBJECT (fontsel));
     gtk_widget_show(widget);
} /* End KeyboardFontSelect */



