/* TEMPLATES.C
 * 
 * Written by David Allen <s2mdalle@titan.vcu.edu>
 * http://opop.nols.com/
 * Part of GTKeyboard, this file is released under the terms of the GNU
 * General Public License
 * See http://www.gnu.org/ for more details.
 */

#define TEMPLATES_C
#include "master.h"

/* Returns the "keyno"th key in row "row" of keyb 
 * This is allocated memory which must be g_free()'d later.
 * This will ALWAYS return allocated memory - it just might always be
 * filled with NoSymbol
 */
KEY *gtkeyboard_keyboard_get_key(KEYBOARD *keyb, int row, int keyno)
{
     KEY *foobar;
     int index;
     int x, findex = 0;

     foobar = g_new0_(KEY, 1);
     foobar->lower_case = NoSymbol;
     foobar->upper_case = NoSymbol;
     foobar->alt_gr     = NoSymbol;

     if(row > MAXIMUM_ROWS)
     {
	  fprintf(stderr,"gtkeyboard_keyboard_get_key:  Row out of range.\n");
	  fflush(stderr);
	  return(foobar);
     } /* End if */

     if(!keyb)
     {
	  fprintf(stderr,"gtkeyboard_keyboard_get_key:  Null keyb.\n");
	  fflush(stderr);
	  return(foobar);
     }

     x = findex = 0;
     while(x < row)
     {
	  /* Add up the number of keys on all lines preceeding the one we're
	   * looking for
	   */
	  findex += keyb->row_values[x];
	  x++;
     }

     index = (findex * 3) + (keyno * 3);

     if(index > ((keyb->keycount * 3)-3))
     {
	  fprintf(stderr,"gtkeyboard_keyboard_get_key():  ");
	  fprintf(stderr,"Illegal index %d of a total keycount of %d (%d).\n",
		  index, keyb->keycount, ((keyb->keycount*3)-3)); 
	  fflush(stderr);
	  return(foobar);
     } /* End if */

     /* Three consecutive KeySyms */
     foobar->lower_case = keyb->syms[index];
     foobar->upper_case = keyb->syms[(index+1)];
     foobar->alt_gr     = keyb->syms[(index+2)];
     foobar->aux_string = (char *)NULL;             /* No auxilliary */

     if(keyb->codes)
	  foobar->code       = keyb->codes[(findex+keyno)];
     else foobar->code       = 0;

     return(foobar);
} /* End gtkeyboard_keyboard_get_key() */

/* Return 1 if we would normally have to press shift to get this key.  
 * Otherwise return 0 
 */
int is_uppercase_key(KEYBOARD *keyb, KeySym i)
{
     int x=0;
     if(!keyb)
     {
#ifdef DEBUGGING
	  fprintf(Q,"is_uppercase_key:  Bad KEYBOARD argument.\n");
	  fflush(Q);
#endif
	  return(0);
     }

     /* We need to know if the KeySym in question needs to be logical or'd 
      * with ShiftMask, so we have to determine if, under the current mapping,
      * we would have to press shift or not to get this key.  Since everything
      * is in keyb->syms in groups of 3, (lower case, uppercase, altgr)
      * then if we find it, and it's in one of the 2nd places, (i.e. uppercase)
      * then return 1.  Else 0.  Since everything is grouped by 3's, 
      * if the position % 3 == 1, then it's good.  
      * Lowercase % 3 => 0
      * uppercase % 3 => 1
      * altgr     % 3 => 2
      */

     for(x=0; x<(keyb->keycount * 3); x++)
     {
	  if(keyb->syms[x] == i)
	  {
	       if((x % 3) == 1) return(1);
	       else             return(0);
	  } /* End if */
     } /* End for */

#ifdef DEBUGGING
     fprintf(Q,"is_uppercase_key():  KeySym not found in keyboard.\n");
     fflush(Q);
#endif
     return(0);
} /* End is_uppercase_key() */

void gtkeyboard_key_set_code(KEY *key, KeyCode input)
{
     if(key)
	  key->code = input;
} /* End gtkeyboard_key_set_code */

int gtkeyboard_keyboard_row_count_items(KEYBOARD *input, int rowno)
{
     if(!input)
     {
	  fprintf(stderr,"gtkeyboard_keyboard_row_count_items:  ");
	  fprintf(stderr,"Invalid keyboard passed.\n"); fflush(stderr);
	  return(0);
     }
     else return(input->row_values[rowno]);
} /* End gtkeyboard_keyboard_row_count_items */

KEY *gtkeyboard_new_key(const KeySym lower, const KeySym upper, 
			const KeySym altgr, const char *alt)
{
     KEY *somekey = g_new0_(KEY, 1);
     somekey->lower_case = lower;
     somekey->upper_case = upper;
     somekey->alt_gr     = altgr;
     somekey->code       = 0;
     somekey->aux_string = g_strdup_(alt);
     return(somekey);
} /* End gtkeyboard_new_key() */

KEY *gtkeyboard_destroy_key(KEY *input)
{
     if(input && input->aux_string)
     {
	  g_free_(input->aux_string);
	  input->aux_string = (char *)NULL;
     }

     if(input)
     {
	  g_free_(input);
	  input = NO_KEY;
     }

     return(NO_KEY);
} /* End gtkeyboard_destroy_key() */

KEYBOARD *gtkeyboard_destroy_keyboard(KEYBOARD *input)
{
     if(!input)
     {
	  fprintf(stderr,
		  "gtkeyboard_destroy_keyboard: Cannot destroy NULL ptr.\n");
	  fflush(stderr);
	  return(NO_KEYBOARD);
     } /* End if */

     if(input->syms)
     {
	  g_free_(input->syms);
	  input->syms = NULL;
     }

     if(input->name)
     {
	  g_free_(input->name);
	  input->name = NULL;
     } /* End if */

     if(input->codes)
     {
	  g_free_(input->codes);
	  input->codes = NULL;
     }

     g_free_(input);
     input = NO_KEYBOARD;

     return(NO_KEYBOARD);
} /* End gtkeyboard_destroy_keyboard() */

static int get_valid_line(FILE *fp, char *mem, const int maxlen)
{
     int x=0;

     mem[0] = '\0';
     
     while(!feof(fp) && mem[0] == '\0')
     {
	  FILE_readline(fp, mem, maxlen);
	  
	  if(mem[0] != '\0')
	  {
	       do
	       {
		    x++;
	       } while(isspace(mem[x]) && (x < strlen(mem)));

	       /* Commented line, so make the first byte null so we loop
		* again.
		*/
	       if(mem[x] && mem[x] == '#')
		    mem[0] = '\0';
	  } /* End if */
     } /* End while */
     
     if(mem[0] == '\0')
	  return(0);
     else return(1);
} /* End get_valid_line() */

KEYBOARD *read_keyboard_template(char *filename)
{
     KEYBOARD *keyb = NO_KEYBOARD;
     FILE *fp = fopen(filename, "r");
     register int x=0, y=0;
     int line_size = 1024;
     int index = 0;
     char linedata[line_size];
     char **tokens;
     char **tofree;
     char *ptr;

     fprintf(Q,"Reading keyboard template at file \"%s\"\n",filename);
     fflush(Q);

     if(!filename)
     {
	  fprintf(stderr,"Readdef error:  NULL filename.\n");
	  fflush(stderr);
	  return(NO_KEYBOARD);
     } /* End if */

     fp = fopen(filename, "r");

     if(!fp)
     {
	  fprintf(stderr,"Error:  Couldn't open %s for reading: %s\n",
		  filename, g_strerror(errno)); 
	  fflush(stderr);
	  return(NO_KEYBOARD);
     }

     linedata[0] = '\0';

     if(!get_valid_line(fp, linedata, line_size))
     {
	  fprintf(stderr,"Error in file %s:  Couldn't read header values.\n",
		  filename);
	  fflush(stderr); 
	  fclose(fp);
	  return(NO_KEYBOARD);
     }

     keyb = g_new0_(KEYBOARD, 1);

     tofree = g_strsplit(linedata, " ", -1);
     tokens = tofree;

     keyb->keycount = 0;

     for(x=0; x<MAXIMUM_ROWS; x++)
     {
	  if(*tokens)
	       ptr = *tokens++;
	  else ptr = NULL;
	  
	  if(ptr)
	       keyb->row_values[x] = atoi(ptr);
	  else
	  {
	       *tokens = NULL;
	       keyb->row_values[x] = 0;
	  } /* End else */

	  keyb->keycount += keyb->row_values[x];
     } /* End for */
     
     g_strfreev(tofree);     tofree = tokens = NULL;  ptr = NULL;

     /* We now know how many keys we have to allocate, how many lines to read,
      * and all that good stuff.
      *
      * Each key must have 3 syms, (lower case, upper case, and Alt Gr)
      * So allocate 3*keyb->keycount items, and read keyb->keycount lines.
      */
     keyb->syms  = g_new0_(KeySym, (3 * keyb->keycount));
     keyb->codes = g_new0_(KeyCode, keyb->keycount);
     keyb->name  = g_strdup_(filename);  /* Save the name of the keyboard */

     for(x=0; x<keyb->keycount; x++)
     {
	  keyb->codes[x] = 0;  /* Initialize that keycode since we're already
				* paying the price of the loop and it needs
				* to be done.
				*/

	  if(!get_valid_line(fp, linedata, line_size))
	  {
	       fprintf(stderr,"Error reading file %s:  Bad line %d.\n",
		       filename, (x+1)); fflush(stderr);
	       keyb = gtkeyboard_destroy_keyboard(keyb);
	       fclose(fp);
	       return(NO_KEYBOARD);
	  } /* End if */

	  tokens = tofree = g_strsplit(linedata, " ", -1);
	  
	  for(y=0; y<3; y++)
	  {
	       if(*tokens)
		    ptr = *tokens++;
	       else ptr = NULL;

	       index = (x * 3) + y;

	       if(ptr)
	       {
		    /* Translate a string into a KeySym */
		    keyb->syms[index] = XStringToKeysym(ptr);

		    /* Error check that KeySym */
		    if(!keyb->syms[index] || keyb->syms[index] == NoSymbol)
		    {
			 keyb->syms[index] = NoSymbol;
			 fprintf(stderr,"GTKeyboard Warning: Bad sym: \"%s\" ",
				 ptr); 
			 fprintf(stderr,"in key #%d position %d.\n",x,y);
			 fflush(stderr);
		    } /* End if */
	       } /* End if */
	       else 
	       {
		    /* We had a NULL string pointer.  This is OK, but 
		     * warn the user.
		     */
		    fprintf(stderr,"GTKeyboard Warning:  No ");
		    switch(y){
		    case 1: fprintf(stderr,"1st "); break;
		    case 2: fprintf(stderr,"2nd "); break;
		    case 3: fprintf(stderr,"3rd "); break;
		    default: fprintf(stderr,"%dth ",y); break;
		    }
		    fprintf(stderr,"KeySym found for key #%d\n",x);
		    fflush(stderr);
		    keyb->syms[index] = NoSymbol;
	       } /* End else */
	  } /* End for */

	  if(ptr)
	       ptr = *tokens++;

	  /* Grab the KeyCode if it's there */
	  if(ptr)
	       keyb->codes[x] = atoi(ptr);

	  g_strfreev(tofree);
     } /* End for */

     fclose(fp);

#ifdef DEBUGGING
     fprintf(Q,"Successful reading of keyboard completed:  %d total keys.\n",
	     keyb->keycount); fflush(Q);
#endif
     return(keyb);
} /* End read_keyboard_template() */

